package org.freecompany.brimstone.module;

import java.io.IOException;
import java.net.MalformedURLException;
import java.net.URL;
import java.net.URLClassLoader;
import java.util.Arrays;
import java.util.Map;
import java.util.HashMap;
import java.util.logging.*;

import static java.util.logging.Level.*;

public class FlatClassLoader extends URLClassLoader {

	protected static Logger logger = Logger.getLogger( FlatClassLoader.class.getName());

	public FlatClassLoader( final URL url) {
		super( new URL[ 0]);
		addUrl( url);
	}

	public FlatClassLoader( final URL[] urls) {
		super( new URL[ 0]);
		for ( URL url : urls) addUrl( url);
	}

	public FlatClassLoader( final Module module) {
		super( new URL[ 0]);
		addModule( module);
	}

	public void addUrl( final URL url) {
		logger.log( FINE, "Adding url '" + url + "'.");
		try {
			final Module module = ( Module) url.openConnection().getContent( new Class[] { Module.class});
			if ( module != null) addModule( module);
			else addURL(  url);
		} catch ( IOException e) {
			throw new IllegalArgumentException( "Error occured while loading resource '" + url + "'.", e);
		}
	}

	public void addModule( final Module module) {
		logger.log( FINE, "Adding module '" + module + "'.");
		try {
			for ( URL url : module.getComponents()) addUrl( url);
			for ( URL url : module.getDependencies()) addUrl( url);
		} catch ( MalformedURLException e) {
			throw new IllegalArgumentException( "The provided module file has invalid entries.", e);
		}
	}
}
